
From: Rusty Russell <rusty@rustcorp.com.au>

Introduce _sinittext and _einittext (cf. _stext and _etext), so kallsyms
includes __init functions.

TODO: Use huffman name compression and 16-bit offsets (see IDE
oopser patch)




 arch/alpha/vmlinux.lds.S     |    6 +++++-
 arch/arm/vmlinux-armo.lds.in |    2 ++
 arch/arm/vmlinux-armv.lds.in |    2 ++
 arch/i386/vmlinux.lds.S      |    6 +++++-
 arch/ia64/vmlinux.lds.S      |    6 +++++-
 arch/m68k/vmlinux-std.lds    |    6 +++++-
 arch/m68k/vmlinux-sun3.lds   |    6 +++++-
 arch/m68knommu/vmlinux.lds.S |    2 ++
 arch/parisc/vmlinux.lds.S    |    6 +++++-
 arch/ppc/vmlinux.lds.S       |    6 +++++-
 arch/ppc64/vmlinux.lds.S     |    6 +++++-
 arch/s390/vmlinux.lds.S      |    6 +++++-
 arch/s390x/vmlinux.lds.S     |    6 +++++-
 arch/sparc/vmlinux.lds.S     |    6 +++++-
 arch/sparc64/vmlinux.lds.S   |    6 +++++-
 arch/v850/vmlinux.lds.S      |    4 ++++
 arch/x86_64/vmlinux.lds.S    |    6 +++++-
 kernel/extable.c             |    6 +++++-
 kernel/kallsyms.c            |   21 +++++++++++++++++++--
 scripts/kallsyms.c           |   12 +++++++-----
 20 files changed, 106 insertions(+), 21 deletions(-)

diff -puN arch/alpha/vmlinux.lds.S~init-sections-in-kallsyms arch/alpha/vmlinux.lds.S
--- 25/arch/alpha/vmlinux.lds.S~init-sections-in-kallsyms	2003-04-02 00:42:56.000000000 -0800
+++ 25-akpm/arch/alpha/vmlinux.lds.S	2003-04-02 00:42:56.000000000 -0800
@@ -32,7 +32,11 @@ SECTIONS
   /* Will be freed after init */
   . = ALIGN(8192);				/* Init code and data */
   __init_begin = .;
-  .init.text : { *(.init.text) }
+  .init.text : { 
+	_sinittext = .;
+	*(.init.text)
+	_einittext = .;
+  }
   .init.data : { *(.init.data) }
 
   . = ALIGN(16);
diff -puN arch/arm/vmlinux-armo.lds.in~init-sections-in-kallsyms arch/arm/vmlinux-armo.lds.in
--- 25/arch/arm/vmlinux-armo.lds.in~init-sections-in-kallsyms	2003-04-02 00:42:56.000000000 -0800
+++ 25-akpm/arch/arm/vmlinux-armo.lds.in	2003-04-02 00:42:56.000000000 -0800
@@ -14,7 +14,9 @@ SECTIONS
 	.init : {			/* Init code and data		*/
 		_stext = .;
 		__init_begin = .;
+			_sinittext = .;
 			*(.init.text)
+			_einittext = .;
 		__proc_info_begin = .;
 			*(.proc.info)
 		__proc_info_end = .;
diff -puN arch/arm/vmlinux-armv.lds.in~init-sections-in-kallsyms arch/arm/vmlinux-armv.lds.in
--- 25/arch/arm/vmlinux-armv.lds.in~init-sections-in-kallsyms	2003-04-02 00:42:56.000000000 -0800
+++ 25-akpm/arch/arm/vmlinux-armv.lds.in	2003-04-02 00:42:56.000000000 -0800
@@ -18,7 +18,9 @@ SECTIONS
 	.init : {			/* Init code and data		*/
 		_stext = .;
 		__init_begin = .;
+			_sinittext = .;
 			*(.init.text)
+			_einittext = .;
 		__proc_info_begin = .;
 			*(.proc.info)
 		__proc_info_end = .;
diff -puN arch/i386/vmlinux.lds.S~init-sections-in-kallsyms arch/i386/vmlinux.lds.S
--- 25/arch/i386/vmlinux.lds.S~init-sections-in-kallsyms	2003-04-02 00:42:56.000000000 -0800
+++ 25-akpm/arch/i386/vmlinux.lds.S	2003-04-02 00:42:56.000000000 -0800
@@ -54,7 +54,11 @@ SECTIONS
   /* will be freed after init */
   . = ALIGN(4096);		/* Init code and data */
   __init_begin = .;
-  .init.text : { *(.init.text) }
+  .init.text : { 
+	_sinittext = .;
+	*(.init.text)
+	_einittext = .;
+  }
   .init.data : { *(.init.data) }
   . = ALIGN(16);
   __setup_start = .;
diff -puN arch/ia64/vmlinux.lds.S~init-sections-in-kallsyms arch/ia64/vmlinux.lds.S
--- 25/arch/ia64/vmlinux.lds.S~init-sections-in-kallsyms	2003-04-02 00:42:56.000000000 -0800
+++ 25-akpm/arch/ia64/vmlinux.lds.S	2003-04-02 00:42:56.000000000 -0800
@@ -96,7 +96,11 @@ SECTIONS
   . = ALIGN(PAGE_SIZE);
   __init_begin = .;
   .init.text : AT(ADDR(.init.text) - PAGE_OFFSET)
-	{ *(.init.text) }
+	{
+	  _sinittext = .;
+	  *(.init.text)
+	  _einittext = .;
+	}
 
   .init.data : AT(ADDR(.init.data) - PAGE_OFFSET)
 	{ *(.init.data) }
diff -puN arch/m68knommu/vmlinux.lds.S~init-sections-in-kallsyms arch/m68knommu/vmlinux.lds.S
--- 25/arch/m68knommu/vmlinux.lds.S~init-sections-in-kallsyms	2003-04-02 00:42:56.000000000 -0800
+++ 25-akpm/arch/m68knommu/vmlinux.lds.S	2003-04-02 00:42:56.000000000 -0800
@@ -282,7 +282,9 @@ SECTIONS {
 	.init : {
 		. = ALIGN(4096);
 		__init_begin = .;
+		_sinittext = .;
 		*(.init.text)
+		_einittext = .;
 		*(.init.data)
 		. = ALIGN(16);
 		__setup_start = .;
diff -puN arch/m68k/vmlinux-std.lds~init-sections-in-kallsyms arch/m68k/vmlinux-std.lds
--- 25/arch/m68k/vmlinux-std.lds~init-sections-in-kallsyms	2003-04-02 00:42:56.000000000 -0800
+++ 25-akpm/arch/m68k/vmlinux-std.lds	2003-04-02 00:42:56.000000000 -0800
@@ -40,7 +40,11 @@ SECTIONS
   /* will be freed after init */
   . = ALIGN(4096);		/* Init code and data */
   __init_begin = .;
-  .init.text : { *(.init.text) }
+  .init.text : { 
+	_sinittext = .;
+	*(.init.text)
+	_einittext = .;
+  }
   .init.data : { *(.init.data) }
   . = ALIGN(16);
   __setup_start = .;
diff -puN arch/m68k/vmlinux-sun3.lds~init-sections-in-kallsyms arch/m68k/vmlinux-sun3.lds
--- 25/arch/m68k/vmlinux-sun3.lds~init-sections-in-kallsyms	2003-04-02 00:42:56.000000000 -0800
+++ 25-akpm/arch/m68k/vmlinux-sun3.lds	2003-04-02 00:42:56.000000000 -0800
@@ -34,7 +34,11 @@ SECTIONS
   /* will be freed after init */
   . = ALIGN(8192);	/* Init code and data */
 __init_begin = .;
-  	.init.text : { *(.init.text) }
+	.init.text : { 
+		_sinittext = .;
+		*(.init.text)
+		_einittext = .;
+	}
   	.init.data : { *(.init.data) }
 	. = ALIGN(16);
 	__setup_start = .;
diff -puN arch/parisc/vmlinux.lds.S~init-sections-in-kallsyms arch/parisc/vmlinux.lds.S
--- 25/arch/parisc/vmlinux.lds.S~init-sections-in-kallsyms	2003-04-02 00:42:56.000000000 -0800
+++ 25-akpm/arch/parisc/vmlinux.lds.S	2003-04-02 00:42:56.000000000 -0800
@@ -53,7 +53,11 @@ SECTIONS
 
   . = ALIGN(16384);
   __init_begin = .;
-  .init.text : { *(.init.text) }
+  .init.text : { 
+	_sinittext = .;
+	*(.init.text)
+	_einittext = .;
+  }
   .init.data : { *(.init.data) }
   . = ALIGN(16);
   __setup_start = .;
diff -puN arch/ppc64/vmlinux.lds.S~init-sections-in-kallsyms arch/ppc64/vmlinux.lds.S
--- 25/arch/ppc64/vmlinux.lds.S~init-sections-in-kallsyms	2003-04-02 00:42:56.000000000 -0800
+++ 25-akpm/arch/ppc64/vmlinux.lds.S	2003-04-02 00:42:56.000000000 -0800
@@ -77,7 +77,11 @@ SECTIONS
   /* will be freed after init */
   . = ALIGN(4096);
   __init_begin = .;
-  .init.text : { *(.init.text) }
+  .init.text : { 
+	_sinittext = .;
+	*(.init.text)
+	_einittext = .;
+  }
   .init.data : { *(.init.data) }
   . = ALIGN(16);
   __setup_start = .;
diff -puN arch/ppc/vmlinux.lds.S~init-sections-in-kallsyms arch/ppc/vmlinux.lds.S
--- 25/arch/ppc/vmlinux.lds.S~init-sections-in-kallsyms	2003-04-02 00:42:56.000000000 -0800
+++ 25-akpm/arch/ppc/vmlinux.lds.S	2003-04-02 00:42:56.000000000 -0800
@@ -78,7 +78,11 @@ SECTIONS
 
   . = ALIGN(4096);
   __init_begin = .;
-  .init.text : { *(.init.text) }
+  .init.text : { 
+	_sinittext = .;
+	*(.init.text)
+	_einittext = .;
+  }
   .init.data : { 
     *(.init.data);
     __vtop_table_begin = .;
diff -puN arch/s390/vmlinux.lds.S~init-sections-in-kallsyms arch/s390/vmlinux.lds.S
--- 25/arch/s390/vmlinux.lds.S~init-sections-in-kallsyms	2003-04-02 00:42:56.000000000 -0800
+++ 25-akpm/arch/s390/vmlinux.lds.S	2003-04-02 00:42:56.000000000 -0800
@@ -58,7 +58,11 @@ SECTIONS
   /* will be freed after init */
   . = ALIGN(4096);		/* Init code and data */
   __init_begin = .;
-  .init.text : { *(.init.text) }
+  .init.text : { 
+	_sinittext = .;
+	*(.init.text)
+	_einittext = .;
+  }
   .init.data : { *(.init.data) }
   . = ALIGN(256);
   __setup_start = .;
diff -puN arch/s390x/vmlinux.lds.S~init-sections-in-kallsyms arch/s390x/vmlinux.lds.S
--- 25/arch/s390x/vmlinux.lds.S~init-sections-in-kallsyms	2003-04-02 00:42:56.000000000 -0800
+++ 25-akpm/arch/s390x/vmlinux.lds.S	2003-04-02 00:42:56.000000000 -0800
@@ -58,7 +58,11 @@ SECTIONS
   /* will be freed after init */
   . = ALIGN(4096);		/* Init code and data */
   __init_begin = .;
-  .init.text : { *(.init.text) }
+  .init.text : { 
+	_sinittext = .;
+	*(.init.text)
+	_einittext = .;
+  }
   .init.data : { *(.init.data) }
   . = ALIGN(256);
   __setup_start = .;
diff -puN arch/sparc64/vmlinux.lds.S~init-sections-in-kallsyms arch/sparc64/vmlinux.lds.S
--- 25/arch/sparc64/vmlinux.lds.S~init-sections-in-kallsyms	2003-04-02 00:42:56.000000000 -0800
+++ 25-akpm/arch/sparc64/vmlinux.lds.S	2003-04-02 00:42:56.000000000 -0800
@@ -41,7 +41,11 @@ SECTIONS
 
   . = ALIGN(8192);
   __init_begin = .;
-  .init.text : { *(.init.text) }
+  .init.text : { 
+	_sinittext = .;
+	*(.init.text)
+	_einittext = .;
+  }
   .init.data : { *(.init.data) }
   . = ALIGN(16);
   __setup_start = .;
diff -puN arch/sparc/vmlinux.lds.S~init-sections-in-kallsyms arch/sparc/vmlinux.lds.S
--- 25/arch/sparc/vmlinux.lds.S~init-sections-in-kallsyms	2003-04-02 00:42:56.000000000 -0800
+++ 25-akpm/arch/sparc/vmlinux.lds.S	2003-04-02 00:42:56.000000000 -0800
@@ -34,7 +34,11 @@ SECTIONS
 
   . = ALIGN(4096);
   __init_begin = .;
-  .init.text : { *(.init.text) }
+  .init.text : { 
+	_sinittext = .;
+	*(.init.text)
+	_einittext = .;
+  }
   __init_text_end = .;
   .init.data : { *(.init.data) }
   . = ALIGN(16);
diff -puN arch/v850/vmlinux.lds.S~init-sections-in-kallsyms arch/v850/vmlinux.lds.S
--- 25/arch/v850/vmlinux.lds.S~init-sections-in-kallsyms	2003-04-02 00:42:56.000000000 -0800
+++ 25-akpm/arch/v850/vmlinux.lds.S	2003-04-02 00:42:56.000000000 -0800
@@ -105,7 +105,9 @@
 #define RAMK_INIT_CONTENTS_NO_END					      \
 		. = ALIGN (4096) ;					      \
 		__init_start = . ;					      \
+			_sinittext = .;					      \
 			*(.init.text)	/* 2.5 convention */		      \
+			_einittext = .;					      \
 			*(.init.data)					      \
 			*(.text.init)	/* 2.4 convention */		      \
 			*(.data.init)					      \
@@ -125,7 +127,9 @@
 /* The contents of `init' section for a ROM-resident kernel which
    should go into ROM.  */	
 #define ROMK_INIT_ROM_CONTENTS						      \
+			_sinittext = .;					      \
 			*(.init.text)	/* 2.5 convention */		      \
+			_einittext = .;					      \
 			*(.text.init)	/* 2.4 convention */		      \
 		INITCALL_CONTENTS					      \
 		INITRAMFS_CONTENTS
diff -puN arch/x86_64/vmlinux.lds.S~init-sections-in-kallsyms arch/x86_64/vmlinux.lds.S
--- 25/arch/x86_64/vmlinux.lds.S~init-sections-in-kallsyms	2003-04-02 00:42:56.000000000 -0800
+++ 25-akpm/arch/x86_64/vmlinux.lds.S	2003-04-02 00:42:56.000000000 -0800
@@ -78,7 +78,11 @@ SECTIONS
 
   . = ALIGN(4096);		/* Init code and data */
   __init_begin = .;
-  .init.text : { *(.init.text) }
+  .init.text : { 
+	_sinittext = .;
+	*(.init.text)
+	_einittext = .;
+  }
   .init.data : { *(.init.data) }
   . = ALIGN(16);
   __setup_start = .;
diff -puN kernel/extable.c~init-sections-in-kallsyms kernel/extable.c
--- 25/kernel/extable.c~init-sections-in-kallsyms	2003-04-02 00:42:56.000000000 -0800
+++ 25-akpm/kernel/extable.c	2003-04-02 00:44:39.000000000 -0800
@@ -20,7 +20,7 @@
 
 extern const struct exception_table_entry __start___ex_table[];
 extern const struct exception_table_entry __stop___ex_table[];
-extern char _stext[], _etext[];
+extern char _stext[], _etext[], _sinittext[], _einittext[];
 
 /* Given an address, look for it in the exception tables. */
 const struct exception_table_entry *search_exception_tables(unsigned long addr)
@@ -39,5 +39,9 @@ int kernel_text_address(unsigned long ad
 	    addr <= (unsigned long)_etext)
 		return 1;
 
+	if (addr >= (unsigned long)_sinittext &&
+	    addr <= (unsigned long)_einittext)
+		return 1;
+
 	return module_text_address(addr) != NULL;
 }
diff -puN kernel/kallsyms.c~init-sections-in-kallsyms kernel/kallsyms.c
--- 25/kernel/kallsyms.c~init-sections-in-kallsyms	2003-04-02 00:42:56.000000000 -0800
+++ 25-akpm/kernel/kallsyms.c	2003-04-02 00:42:56.000000000 -0800
@@ -15,7 +15,22 @@ extern unsigned long kallsyms_num_syms _
 extern char kallsyms_names[] __attribute__((weak));
 
 /* Defined by the linker script. */
-extern char _stext[], _etext[];
+extern char _stext[], _etext[], _sinittext[], _einittext[];
+
+static inline int is_kernel_inittext(unsigned long addr)
+{
+	if (addr >= (unsigned long)_sinittext
+	    && addr <= (unsigned long)_einittext)
+		return 1;
+	return 0;
+}
+
+static inline int is_kernel_text(unsigned long addr)
+{
+	if (addr >= (unsigned long)_stext && addr <= (unsigned long)_etext)
+		return 1;
+	return 0;
+}
 
 /* Lookup an address.  modname is set to NULL if it's in the kernel. */
 const char *kallsyms_lookup(unsigned long addr,
@@ -31,7 +46,7 @@ const char *kallsyms_lookup(unsigned lon
 	namebuf[127] = 0;
 	namebuf[0] = 0;
 
-	if (addr >= (unsigned long)_stext && addr <= (unsigned long)_etext) {
+	if (is_kernel_text(addr) || is_kernel_inittext(addr)) {
 		unsigned long symbol_end;
 		char *name = kallsyms_names;
 
@@ -52,6 +67,8 @@ const char *kallsyms_lookup(unsigned lon
 		/* Base symbol size on next symbol. */
 		if (best + 1 < kallsyms_num_syms)
 			symbol_end = kallsyms_addresses[best + 1];
+		else if (is_kernel_inittext(addr))
+			symbol_end = (unsigned long)_einittext;
 		else
 			symbol_end = (unsigned long)_etext;
 
diff -puN scripts/kallsyms.c~init-sections-in-kallsyms scripts/kallsyms.c
--- 25/scripts/kallsyms.c~init-sections-in-kallsyms	2003-04-02 00:42:56.000000000 -0800
+++ 25-akpm/scripts/kallsyms.c	2003-04-02 00:42:56.000000000 -0800
@@ -21,7 +21,7 @@ struct sym_entry {
 
 static struct sym_entry *table;
 static int size, cnt;
-static unsigned long long _stext, _etext;
+static unsigned long long _stext, _etext, _sinittext, _einittext;
 
 static void
 usage(void)
@@ -51,10 +51,8 @@ read_symbol(FILE *in, struct sym_entry *
 static int
 symbol_valid(struct sym_entry *s)
 {
-	if (s->addr < _stext)
-		return 0;
-
-	if (s->addr > _etext)
+	if ((s->addr < _stext || s->addr > _etext)
+	    && (s->addr < _sinittext || s->addr > _einittext))
 		return 0;
 
 	if (strstr(s->sym, "_compiled."))
@@ -85,6 +83,10 @@ read_map(FILE *in)
 			_stext = table[i].addr;
 		if (strcmp(table[i].sym, "_etext") == 0)
 			_etext = table[i].addr;
+		if (strcmp(table[i].sym, "_sinittext") == 0)
+			_sinittext = table[i].addr;
+		if (strcmp(table[i].sym, "_einittext") == 0)
+			_einittext = table[i].addr;
 	}
 }
 

_
